from . import __version__
from .config import config_data
from .about import About
from .notes import save_note
from .assets import get_logo_path_high_res
from platform import system
import tkinter
import tkinter.font
from pathlib import Path
import sys

class Gui(tkinter.Tk):
    def __init__(self, config: config_data) -> None:
        super().__init__()

        self.user_config: config_data = config

        self.bind_all('<Control-Return>', self.save_note_and_exit)
        self.resizable(False, False)
        self.title('New Note')

        # Set the logo of the app and make it MacOS compatible
        logo_path: Path = get_logo_path_high_res()
        logo: tkinter.Image = tkinter.Image('photo', file=f'{logo_path}')
        # Disable type checking because _w is a internal Tkinter var and cant be detected by type checkers
        self.tk.call('wm','iconphoto', self._w, logo)  # type: ignore

        # Set the widgets
        self.set_widgets()
        self.input.focus_set()

        # Start the loops
        self.scrollbar_loop()
        self.title_loop()

        # Add about button in not Darwin OS
        if system() != 'Darwin':
            self.set_about_button()

        # After all the widgets are drew position the window
        self.set_window_position()

        # Set about menu
        self.about_menu_open: bool = False
        self.createcommand('tkAboutDialog', self.display_about_menu)

        # Redefine the exit custom to completely exit the app
        self.protocol('WM_DELETE_WINDOW', self.exit)

        self.mainloop()
    
    def set_widgets(self) -> None:

        font_size: tkinter.font.Font = tkinter.font.Font(size=20)

        # Input widget
        self.input_frame: tkinter.Frame = tkinter.Frame(self)
        self.input_frame.pack(
            side=tkinter.TOP
        )

        self.input = tkinter.Text(
            self.input_frame,
            highlightthickness=0,
            padx=20,
            pady=20,
            font= font_size,
            height=9,
            width=30,
            )
        self.input.pack(side=tkinter.LEFT)

        # Scrollbar widget
        self.scrollbar = tkinter.Scrollbar(self.input_frame)

        # Set the relation between the textbox and the scrollbar
        self.input.config(yscrollcommand=self.scrollbar.set)
        self.scrollbar.config(command=self.input.yview)

        # Clean the menu bar
        self.menu = tkinter.Menu()
        self.config(menu=self.menu)

    def title_loop(self):
        """
        Updates the title text according the first line in the textbox.
        """
        new_title: str = self.input.get('1.0', '1.end')
        if len(new_title) >= 20:
            new_title = new_title[:17] + '...'

        if new_title != '':
            self.title(new_title)
        else:
            self.title('New Note')

        self.after(1, self.title_loop)

    def scrollbar_loop(self):
        """
        Check if the number of lines has exceeded the maximum of the textbox and show or hide a scrollbar as appropriate.
        """
        number_of_lines: int = self.input.get('1.0', 'end-1c').count('\n')

        if number_of_lines > 8:
            self.scrollbar.pack(side=tkinter.RIGHT, fill='y')
        else:
            self.scrollbar.pack_forget()

        self.after(1, self.scrollbar_loop)

    def set_window_position(self):
        self.attributes('-alpha', 0.0)

        # Force an update to get the real size of the window
        self.update()
        
        self.window_width: int = self.winfo_width()
        self.window_height: int = self.winfo_height()

        screen_width: int = self.winfo_screenwidth()
        screen_height: int = self.winfo_screenheight()

        # Calculate the coordinates for the top left of the window
        calculated_width: int = int(screen_width/2 - self.window_width/2)
        calculated_height: int = int(screen_height/2 - self.window_height/2)

        # Apply the correct position of the window
        self.geometry(f'{self.window_width}x{self.window_height}+{calculated_width}+{calculated_height}')

        # Detect when the window has finally moved and make it visible
        self.bind("<Configure>", self.make_window_visible)

    def make_window_visible(self, event: tkinter.Event):
        self.attributes('-alpha', 1.0)
        self.bind("<Configure>", None)

    def display_about_menu(self, event: tkinter.Event | None = None) -> None:
        if self.about_menu_open == False:
            self.about_menu_open = True
            self.about_menu: About = About(self.winfo_x(), self.winfo_y(), self.window_width, self.window_height)
            self.about_menu.protocol('WM_DELETE_WINDOW', self.close_about_menu)

    def close_about_menu(self) -> None:
        self.about_menu_open = False
        self.about_menu.destroy()

    def set_about_button(self) -> None:
        self.about_frame: tkinter.Frame = tkinter.Frame(self)
        self.about_frame.pack(
            side=tkinter.BOTTOM
        )

        self.about_button: tkinter.Label = tkinter.Label(
            self,
            text='About YAQN',
        )

        self.about_button.pack()

        self.about_button.bind('<Button-1>', self.display_about_menu)

    def save_note_and_exit(self, event: tkinter.Event) -> None:
        save_note(
            self.input.get('1.0', '1.end'),
            self.input.get('1.0', 'end-1c'),
            self.user_config
        )
        
        self.exit()

    def exit(self):
        sys.exit(0)



from .assets import get_logo_path_low_res, get_logo_path_high_res
from . import __version__
from pathlib import Path
import tkinter

class About(tkinter.Toplevel):
    def __init__(
            self,
            parent_window_position_x: int,
            parent_window_position_y: int, 
            parent_window_size_x: int,
            parent_window_size_y: int
        ) -> None:

        super().__init__()

        # Set the logo of the app and make it MacOS compatible
        logo_path: Path = get_logo_path_high_res()
        logo: tkinter.Image = tkinter.Image('photo', file=f'{logo_path}')
        # Disable type checking because _w is a internal Tkinter var and cant be detected by type checkers
        self.tk.call('wm','iconphoto', self._w, logo)  # type: ignore

        self.set_widgets()
        self.set_window_position(
            parent_window_position_x,
            parent_window_position_y,
            parent_window_size_x,
            parent_window_size_y
        )

    def set_widgets(self) -> None:
        self.title('About YAQN')
        self.resizable(False, False)

        # Set the logo of YAQN as the first half of the window
        self.image_frame: tkinter.Frame = tkinter.Frame(self)
        self.image_frame.pack(side=tkinter.TOP)

        # Define the logo as a tkinter image object
        logo_path: Path = get_logo_path_low_res()
        self.logo_image: tkinter.PhotoImage = tkinter.PhotoImage(file=logo_path)

        # Put the image object to a label
        self.image_label: tkinter.Label = tkinter.Label(
            self,
            image=self.logo_image
        )
        self.image_label.pack(
            padx=50,
            pady=(10, 5)
        )

        # Set the texts
        self.text_frame: tkinter.Frame = tkinter.Frame(self)
        self.text_frame.pack()

        # Set the title of the app and make it bold
        font_title: tkinter.font.Font = tkinter.font.Font(weight='bold')

        self.title: tkinter.Label = tkinter.Label(
            self.text_frame,
            text='YAQN',
            font=font_title
        )
        self.title.pack()

        font_subtitle: tkinter.font.Font = tkinter.font.Font(size=14)

        # Set the subtitle
        self.subtitle: tkinter.Label = tkinter.Label(
            self.text_frame,
            text='Yet Another Quick Note',
            font=font_subtitle
        )
        self.subtitle.pack()

        # Set the version number and make it smaller
        font_little: tkinter.font.Font = tkinter.font.Font(size=12)

        self.version: tkinter.Label = tkinter.Label(
            self.text_frame,
            text=f'Version {__version__}',
            font=font_little
        )
        self.version.pack(
            pady=(0, 5)
        )

        # Set the created by section
        self.created_by_frame: tkinter.Frame = tkinter.Frame(self)
        self.created_by_frame.pack(side=tkinter.BOTTOM)

        self.kutu_label: tkinter.Label = tkinter.Label(
            self.created_by_frame,
            text='Created with ♥ by\nKutu (@kutu-dev)',
            font=font_little
        )
        self.kutu_label.pack()

        self.other_authors_label: tkinter.Label = tkinter.Label(
            self.created_by_frame,
            text='App icon created by\n\'vladlucha\' in MacOS Icons',
            font=font_little
        )
        self.other_authors_label.pack(
            pady=(0, 25)
        )

    def set_window_position(
        self,
        parent_window_position_x: int,
        parent_window_position_y: int, 
        parent_window_size_x: int,
        parent_window_size_y: int
    ) -> None:

        # Make the window transparent and after update it to avoid seeing the window change its position
        self.attributes('-alpha', 0.0)
        self.update()

        window_width: int = self.winfo_width()
        window_height: int = self.winfo_height()

        parent_window_center_x: int = parent_window_position_x + parent_window_size_x/2
        parent_window_center_y: int = parent_window_position_y + parent_window_size_y/2

        # Calculate the coordinates for the top left of the window
        calculated_width: int = int(parent_window_center_x - window_width/2)
        calculated_height: int = int(parent_window_center_y - window_height/2)

        # Apply the correct position of the window
        self.geometry(f'{window_width}x{window_height}+{calculated_width}+{calculated_height}')

        # Detect when the window has finally moved and make it visible
        self.bind("<Configure>", self.make_window_visible)

    def make_window_visible(self, event: tkinter.Event):
        self.attributes('-alpha', 1.0)
        self.bind("<Configure>", None)



from pathlib import Path
from dataclasses import dataclass
from .terminal import warn
import platform
import tomllib
import typing

# Define the default paths
DEFAULT_NOTES_PATH: Path = Path(Path.home(), 'Documents', 'notes')
DEFAULT_EXTENSION: str = 'md'

# Structure of the config
@dataclass
class config_data():
    notes_dir_path: Path
    extension: str
    no_whitespaces: bool
    no_uppercase: bool
    no_firstline: bool

def init_config(custom_path: Path | None = None) -> Path:
    """
    Check if the config path and file exist and configure a custom path if it is given.
    """
    config_path: Path = get_default_config_path()

    if custom_path is not None:
        config_path = custom_path

    if not config_path.is_dir():
        config_path.mkdir()

    yaqn_path: Path = Path(config_path, 'yaqn')
    yaqn_file: Path = Path(yaqn_path, 'config.toml')

    if not yaqn_path.is_dir():
        yaqn_path.mkdir()

    if not yaqn_file.is_file():
        yaqn_file.touch()
        regenerate_config(yaqn_file, True)

    return yaqn_file

def get_default_config_path() -> Path:
    if platform.system() == 'Windows':
        return Path(Path.home(), 'AppData', 'Roaming')
    else:
        return Path(Path.home(), '.config')

def check_config(config_path: Path) -> None:
    """
    Check if the config file is valid
    """
    with open(config_path, 'rb') as config:
        # Check if the config is a valid toml file
        loaded_config: dict
        try:
            loaded_config = tomllib.load(config)
        except tomllib.TOMLDecodeError:
            regenerate_config(config_path)
            # Load the config again and check the structure
            return

        # Check if the config is following the expected structure
        match loaded_config:
            case {
                'notes_path': str(),
                'extension': str(),
                'no_whitespaces': bool(),
                'no_uppercase': bool(),
                'no_firstline': bool()
                }:
                pass
            case _:
                regenerate_config(config_path)


def regenerate_config(config_path: Path, silent: bool = False) -> None:
    """
    Rewrite the config file with all the defaults.
    """

    if not config_path.is_file():
        warn('Not config file was found, use \'yaqn --check\' to create a new one')

    if not silent:
        warn('The config was invalid and it has been restored to its defaults')
    
    # Rewrite all the config file
    with open(config_path, 'w') as config:
        config.writelines([
            f'notes_path = \'{DEFAULT_NOTES_PATH}\'',
            f'\nextension = \'{DEFAULT_EXTENSION}\'',
            f'\nno_whitespaces = false',
            f'\nno_uppercase = false',
            f'\nno_firstline = false',
            f'\n'
        ])

def read_config(custom_path: Path | None = None) -> config_data:
    """
    Read the config file, check it and repair it if is necessary.
    """
    config_path: Path = init_config(custom_path)
    check_config(config_path)

    with open(config_path, 'rb') as config:
        data: dict[str, typing.Any] = tomllib.load(config)

        return config_data(
            Path(data['notes_path']),
            data['extension'],
            data['no_whitespaces'],
            data['no_uppercase'],
            data['no_firstline']
        )

def restore_config(custom_path: Path | None = None) -> None:
    """
    Restore the configuration to its defaults values
    """
    config_path: Path = init_config(custom_path)
    regenerate_config(config_path, True)
